package cc.fascinated.account;

import cc.fascinated.Aetheria;
import cc.fascinated.config.Lang;
import cc.fascinated.playercolor.PlayerColorProfile;
import cc.fascinated.utils.Style;
import cc.fascinated.vote.VoteProfile;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.extern.log4j.Log4j2;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;

import java.io.File;
import java.util.UUID;

@Getter
@Log4j2
@EqualsAndHashCode
public class Account {

    /**
     * Profile keys.
     */
    private static final String playerColorProfileId = "playerColorProfile";
    private static final String voteProfileId = "voteProfile";

    /**
     * The UUID of the player.
     */
    @EqualsAndHashCode.Exclude
    private final UUID uuid;

    /**
     * The name of the player.
     */
    private final String name;

    /**
     * The file for this account.
     */
    @EqualsAndHashCode.Exclude
    private final File file;

    /**
     * The configuration for this account.
     */
    @EqualsAndHashCode.Exclude
    private final FileConfiguration config;

    /**
     * Account profiles.
     */
    private final PlayerColorProfile playerColorProfile;
    private final VoteProfile voteProfile;

    /**
     * The last hashcode of the account. This is used to check if any changes
     * have been made to the account. If no changes have been made,
     * the account will not be saved this is to prevent unnecessary saving.
     */
    @EqualsAndHashCode.Exclude
    private int lastHash = -1;

    /**
     * The first time the player joined the server.
     */
    private long firstJoin;

    /**
     * The last time the player logged in.
     */
    private long lastLogin;

    public Account(UUID uuid, String name) {
        //log.info("Loading account for " + uuid);
        boolean newAccount = false;

        this.uuid = uuid;

        File file = new File(Aetheria.INSTANCE.getDataFolder(), "accounts/" + this.uuid + ".yml");
        this.file = file;
        if (!file.exists()) {
            newAccount = true;
            file.getParentFile().mkdirs();
            try {
                file.createNewFile();
            } catch (Exception e) {
                log.warn("Failed to create account file for {}", this.uuid);
                e.printStackTrace();
            }
        }
        config = YamlConfiguration.loadConfiguration(file);

        if (newAccount) {
            this.firstJoin = System.currentTimeMillis();
            this.lastLogin = System.currentTimeMillis();
            log.info("Created new account for {}", this.uuid);
        }

        this.firstJoin = config.getLong("firstJoin");
        this.lastLogin = config.getLong("lastLogin");

        this.lastLogin = System.currentTimeMillis(); // Update last login
        this.name = name; // Update the name

        // Load profiles
        this.playerColorProfile = new PlayerColorProfile(this, this.getProfileSection(playerColorProfileId, false));
        this.voteProfile = new VoteProfile(this, this.getProfileSection(voteProfileId, false));

        //log.info("Loaded account for " + this.uuid);
    }

    /**
     * Save a profile to the configuration.
     *
     * @param profile the profile to save
     * @param key     the key to save the profile under
     */
    private void saveProfile(Profile profile, String key) {
        profile.save(this.getProfileSection(key, true));
    }

    /**
     * Gets the profile section.
     *
     * @param key the key to get the profile section for
     * @return the profile section
     */
    private ConfigurationSection getProfileSection(String key, boolean create) {
        ConfigurationSection section = this.config.getConfigurationSection(key);
        if (section == null && create) {
            section = this.config.createSection(key);
        }
        return section;
    }

    /**
     * Get the bukkit player for this account.
     *
     * @return the player
     */
    public Player getPlayer() {
        return Bukkit.getPlayer(uuid);
    }

    /**
     * Get the offline player for this account.
     *
     * @return the offline player
     */
    public OfflinePlayer getOfflinePlayer() {
        return Bukkit.getOfflinePlayer(uuid);
    }

    /**
     * Check if the player is online.
     *
     * @return if the player is online
     */
    public boolean isOnline() {
        return this.getPlayer() != null;
    }

    /**
     * Send a message to the player.
     *
     * @param component the message to send
     */
    public void sendMessage(Component component) {
        if (this.isOnline()) {
            this.getPlayer().sendMessage(component);
        }
    }

    /**
     * Send a message to the player.
     *
     * @param message the message to send
     */
    public void sendMessage(String message) {
        this.sendMessage(Style.getMiniMessage().deserialize(message));
    }

    /**
     * Send a message to the player with a prefix.
     *
     * @param prefix  the prefix to use
     * @param message the message to send
     */
    public void sendMessage(String prefix, Component message) {
        Component prefixComponent = Style.getMiniMessage().deserialize(Lang.PREFIX_FORMAT.getAsString()
                .replace("%prefix%", prefix.toUpperCase()) // Always ensure the prefix is uppercase
        );
        this.sendMessage(prefixComponent.append(message));
    }

    /**
     * Send a message to the player with a prefix.
     *
     * @param prefix  the prefix to use
     * @param message the message to send
     */
    public void sendMessage(String prefix, String message) {
        this.sendMessage(prefix, Style.getMiniMessage().deserialize(message));
    }

    /**
     * Check if the player is an operator.
     *
     * @return if the player is an operator
     */
    public boolean isOp() {
        return getPlayer().isOp();
    }

    /**
     * Set if the player has the permission.
     *
     * @param permission the permission to check
     * @return if the player has the permission
     */
    public boolean hasPermission(String permission) {
        return getPlayer().hasPermission(permission);
    }

    /**
     * Saves the account to disk.
     *
     * @return true if the account was saved, false otherwise
     */
    public boolean save() {
        long before = System.currentTimeMillis();
        if (this.lastHash == this.hashCode()) {
            return false; // No changes have been made
        }

        this.config.set("firstJoin", this.firstJoin);
        this.config.set("lastLogin", this.lastLogin);

        this.saveProfile(this.playerColorProfile, playerColorProfileId);
        this.saveProfile(this.voteProfile, voteProfileId);

        try {
            this.config.save(this.file); // Save the account to disk
        } catch (Exception ex) {
            log.warn("Failed to save account for {}", this.uuid);
            ex.printStackTrace();
            return false;
        }
        this.lastHash = this.hashCode(); // Update the last hash

        log.info("Saved account for {}({}) in {}ms!", this.name, this.uuid, System.currentTimeMillis() - before);
        return true;
    }
}
