package cc.fascinated.bot;

import cc.fascinated.config.Config;
import cc.fascinated.utils.Manager;
import lombok.Getter;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.OnlineStatus;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.utils.cache.CacheFlag;
import org.bukkit.Bukkit;

import java.util.Timer;
import java.util.TimerTask;

@Getter @Log4j2
public class DiscordBot extends Manager {

    /**
     * How often we should update the Discord presence.
     */
    private final int statusUpdateInterval = 1000 * 60 * 5;

    /**
     * The Discord bot instance.
     */
    @Getter
    private static JDA jda;

    /**
     * The thread that the bot runs on.
     */
    private final Thread thread;

    public DiscordBot() {
        this.thread = new Thread(() -> {
            log.info("Starting Discord bot...");
            JDABuilder builder = JDABuilder.createDefault(Config.DISCORD_TOKEN.getAsString());
            builder.disableCache(
                    CacheFlag.MEMBER_OVERRIDES,
                    CacheFlag.VOICE_STATE,
                    CacheFlag.ACTIVITY
            );

            try {
                jda = builder.build();
                jda.awaitReady();
                log.info("Discord bot started successfully.");
            } catch (Exception ex) {
                log.error("Failed to start Discord bot.", ex);
                return;
            }

            new Timer().scheduleAtFixedRate(new TimerTask() {
                @Override
                public void run() {
                    int playerCount = Bukkit.getOnlinePlayers().size();

                    jda.getPresence().setPresence(OnlineStatus.ONLINE, Activity.watching("over " + playerCount + " players"));
                }
            }, 0, statusUpdateInterval);
        });
        this.thread.setName("DiscordBot");
        this.thread.start();
    }

    /**
     * Sends a message to the Discord channel.
     *
     * @param channel the channel
     * @param message the message
     */
    public static void sendMessage(DiscordChannel channel, String message) {
        TextChannel textChannel = jda.getTextChannelById(channel.getId());
        if (textChannel == null) {
            log.error("Could not find the text channel with the ID " + channel.getId());
            return;
        }
        textChannel.sendMessage(message).queue();
    }

    /**
     * Sends an embed to the Discord channel.
     *
     * @param channel the channel
     * @param embed the embed
     */
    public static void sendEmbed(DiscordChannel channel, EmbedBuilder embed) {
        TextChannel textChannel = jda.getTextChannelById(channel.getId());
        if (textChannel == null) {
            log.error("Could not find the text channel with the ID " + channel.getId());
            return;
        }
        textChannel.sendMessageEmbeds(embed.build()).queue();
    }

    @Override
    public void onAetheriaDisable() {
        log.info("Shutting down Discord bot...");
        jda.shutdownNow();
    }
}
