package cc.fascinated.metrics;

import cc.fascinated.Aetheria;
import cc.fascinated.config.Config;
import cc.fascinated.metrics.impl.server.*;
import cc.fascinated.metrics.impl.system.CpuUsageMetric;
import cc.fascinated.metrics.impl.world.WorldSizeMetric;
import cc.fascinated.utils.Manager;
import com.influxdb.client.InfluxDBClient;
import com.influxdb.client.InfluxDBClientFactory;
import com.influxdb.client.InfluxDBClientOptions;
import com.influxdb.client.WriteApiBlocking;
import com.influxdb.client.domain.WriteConsistency;
import com.influxdb.client.domain.WritePrecision;
import com.influxdb.client.write.Point;
import com.influxdb.client.write.WriteParameters;
import lombok.extern.log4j.Log4j2;
import org.bukkit.Bukkit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Log4j2
public class MetricManager extends Manager {

    private static final List<Metric> metrics = new ArrayList<>();
    private InfluxDBClient influxDB;
    private WriteApiBlocking writeApi;

    public MetricManager() {
        String url = Config.INFLUXDB_URL.getAsString();
        String token = Config.INFLUXDB_TOKEN.getAsString();
        String org = Config.INFLUXDB_ORG.getAsString();
        String bucket = Config.INFLUXDB_BUCKET.getAsString();

        if (url == null || token == null || org == null || bucket == null) {
            log.error("InfluxDB configuration is missing");
            return;
        }
        InfluxDBClientOptions options = InfluxDBClientOptions.builder()
                .url(url)
                .authenticateToken(token.toCharArray())
                .org(org)
                .bucket(bucket)
                .build();

        try {
            influxDB = InfluxDBClientFactory.create(options);
            influxDB.ping(); // test the connection
            writeApi = influxDB.getWriteApiBlocking();
        } catch (Exception e) {
            log.error("Failed to connect to influx", e);
            return;
        }

        // system metrics
        registerMetric(new CpuUsageMetric());

        // world metrics
        registerMetric(new WorldSizeMetric());

        // server metrics
        registerMetric(new TpsMetric());
        registerMetric(new MemoryMetric());
        registerMetric(new PlayerCountMetric());
        registerMetric(new TotalJoinsMetric());
        registerMetric(new EntityCountMetric());
        registerMetric(new LoadedChunksMetric());

        Bukkit.getScheduler().runTaskTimer(Aetheria.INSTANCE, (task) -> this.collectMetrics(), 30 * 20, 30 * 20);
    }

    /**
     * Register a new metric
     *
     * @param metric the metric to register
     */
    public static void registerMetric(Metric metric) {
        metrics.add(metric);
    }

    /**
     * Collect all metrics and write them to influx
     */
    public void collectMetrics() {
        log.info("Collecting metrics...");
        long before = System.currentTimeMillis();
        HashMap<Metric, Long> times = new HashMap<>();

        // collect all the metrics
        List<Point> points = new ArrayList<>();
        for (Metric metric : metrics) {
            points.add(metric.toPoint());
            times.put(metric, System.currentTimeMillis());
        }

        // write the points async
        Aetheria.EXECUTOR.execute(() -> {
            writeApi.writePoints(points, new WriteParameters(WritePrecision.MS, WriteConsistency.ONE));

            log.info("Wrote {} points to influx ({}ms)", points.size(), System.currentTimeMillis() - before);
        });

        // get the top 3 slowest metrics that are above 50ms and log them
        times.entrySet().stream()
                .filter(entry -> System.currentTimeMillis() - entry.getValue() > 50)
                .sorted((entry1, entry2) -> Long.compare(entry2.getValue(), entry1.getValue()))
                .limit(3)
                .forEach(entry -> log.warn("Metric {} took {}ms to collect", entry.getKey().getName(), System.currentTimeMillis() - entry.getValue()));
    }

    @Override
    public void onAetheriaDisable() {
        influxDB.close(); // close the connection
    }
}
